/**
 * @brief           leds management
 * @file            websampling.c
 * @author          <a href="">Dom-IP</a>
 * @dependencies    none
 * @compiler        MPLAB C18 v2.10 or higher <br>
 *                  HITECH PICC-18 V8.35PL3 or higher
 *
 * @section description Description
 **********************************
 * This module contains functions used for the leds management of evaluation boards
 *
 *********************************************************************/

 /*********************************************************************
 * Software License Agreement
 *
 * The software supplied herewith is owned by Benoit Sandre, and is
 * protected under applicable copyright laws.  The code may
 * be modified and can be used free of charge for commercial and non commercial
 * applications. All rights are reserved. Any use in violation of the foregoing
 * restrictions may subject the user to criminal sanctions under applicable laws,
 * as well as to civil liability for the breach of the terms and conditions of this license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN 'AS IS' CONDITION. NO WARRANTIES, WHETHER EXPRESS,
 * IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE
 * COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 **********************************************************************
 * File History
 *
 * 2009-06-01, Dom-IP
 *    - Initial version
 *********************************************************************/
#include <string.h>
#include "projdefs.h"
#include "appcfg.h"
#include "net\tick.h"
#include "net\compiler.h"
#include "leds.h"
#include "net\dns.h"
#include "net\httpclient.h"
#include "teleinfo.h"
#include "websampling.h"

/////////////////////////////////////////////////
//Function prototypes
//NA

////////////////////////////////
//Websampling application variables
////////////////////////////////
BYTE Status=0;
TICK16 tmrSec;
unsigned short nameResolutionCount=0;


//WebSampling application initialization
void appcfgWebSampling() {
	//init Timer
	tmrSec=TickGetSec();
	//force  dns request
	nameResolutionCount=0;
	//relaunch send if task is in standby
	if ( Status>=6) Status=0;
}

//WebSampling application processing
void appWebSampling() {
static BYTE HostName[32];
static BYTE filename[9];
static IP_ADDR HostIP;
static TCP_FILESEND_INFO Req_Http;
BYTE i;

		// DNS Resolution init
		if (Status==0) {
			SetLeds(1);
			
			// Si la rsolution DNS doit tre lance
			if (nameResolutionCount--==0) {
				//Reinit Tmr
				nameResolutionCount=NAME_RESOLUTION_MAX_COUNT;
				
				//Get host name from EEPROM
				for(i=0;i<32;i++) {
					HostName[i]=appcfgGetc(APPCFG_HOST_NAME0+i);
				}
				//string max length is 32 chars
				HostName[31]=0;
				
				// If hostname is defined
				if (HostName[0]!=0) {
					//Launch DNS request
					DNSResolve(HostName);
					//Status update
					Status=1;
				}
				// Otherwise launch teleinfo frame receiving 
				else Status=2;
			}
			// Otherwise launch teleinfo frame receiving 
			else Status=2;
		}
		//Waiting for DNS resolution
		else if (Status==1) {
			//if dns request is ended
			if ( DNSIsResolved(&HostIP)) {
				//Save result in EEPROM
				for(i=0;i<4;i++) {
					appcfgPutc(APPCFG_HOST_IP0+i, HostIP.v[i]);
				}
				//Launch teleinfo frame receiving 
				Status=2;
			}
		}
		//init teleinformation frame reception
		else if (Status==2)  {
			SetLeds(2);
			//start teleinfo frame reception
			START_TELEINFO_RX;
			Status=3;
		}
		//teleinformation frame reception
		else if (Status==3) {
			//if teleinfo reception is ended, launch sending of http request
			i=processTeleInfo();
			if ( (i==TELEINFO_RX_OK)||(i==TELEINFO_RX_KO) ) Status=4;
		}
		//Init HTTP request
		else if (Status==4) {
			SetLeds(4);
			//Initialize http request
			Req_Http.port=HTTP_SERVICE_PORT;
			Req_Http.serverNode.IPAddr.v[0]=appcfgGetc(APPCFG_HOST_IP0);
			Req_Http.serverNode.IPAddr.v[1]=appcfgGetc(APPCFG_HOST_IP1);
			Req_Http.serverNode.IPAddr.v[2]=appcfgGetc(APPCFG_HOST_IP2);
			Req_Http.serverNode.IPAddr.v[3]=appcfgGetc(APPCFG_HOST_IP3);
			Req_Http.sm=0;
			
			//The request data is saved in the file "requete.htp"
			strcpypgm2ram(filename,"httpreq.cgi");
			Status=5;
		}
		//Continue HTTP request
		else if (Status==5) {
			//start or continue http request
			i=sendFileToTcpServer(filename,&Req_Http);
			if (i==0) Status=6;
			else if (i!=0xff) Status=7;
		}
		//http request has been send successfully
		else if (Status==6) {
			SetLeds(8);
		}
		//http request has not been send
		else if (Status==7) {
			SetLeds(16);
		}
		
		//Every 60 seconds, 
		if (  TickGetDiffSec(tmrSec) >= (TICK16)60 ) {
			//reload the seconds timer
			tmrSec+=(TICK16)60;
			//Relaunch send if task is in Standby
			if (Status >=6) Status=0;
		}
}



